Attribute VB_Name = "modSettings"
Option Explicit

'module only for settings

'*** StdCallDLL ***

Private bDLL_LinkAsDLL As Boolean
Private bExportSymbols As Boolean

Private lBaseAddress As Long
Private sExportedSymbols As String
Private sEntryPointName As String

Private bDLL_UsePreLoader As Boolean
Private bDLL_FullLoading As Boolean
Private bDLL_CallMyDllMain As Boolean

Private bDLL_BPSubMain As Boolean
Private bDLL_BPCallThunRTMain As Boolean
Private bDLL_BPPreLoader As Boolean

Public Enum DLL_
    
    LinkAsDll
    ExportSymbols
    
    BaseAddress
    ExportedSymbols
    EntryPointName
    
    UsePreLoader
    FullLoading
    DetectVbIde
    
    CallMyDllMain
    
    BPSubMain
    BPCallThunRTMain
    BPPreloader
    
End Enum

Private Const MOD_NAME As String = "modSettings"

'-------------------
'--- STDCALL DLL ---
'-------------------

'get StdCall DLL settings
'parameter - eDLL - DLL setting
'return -  False/True, long, string

Public Function Get_DLL(ByVal eDLL As DLL_) As String

    Select Case eDLL
        Case DLL_.ExportSymbols
            Get_DLL = bExportSymbols
        Case DLL_.LinkAsDll
            Get_DLL = bDLL_LinkAsDLL
        
        Case DLL_.BaseAddress
            Get_DLL = lBaseAddress
        Case DLL_.ExportedSymbols
            Get_DLL = sExportedSymbols
        Case DLL_.EntryPointName
            Get_DLL = sEntryPointName
        
        Case DLL_.FullLoading
            Get_DLL = bDLL_FullLoading
        Case DLL_.UsePreLoader
            Get_DLL = bDLL_UsePreLoader
        Case DLL_.CallMyDllMain
            Get_DLL = bDLL_CallMyDllMain
            
        Case DLL_.BPCallThunRTMain
            Get_DLL = bDLL_BPCallThunRTMain
        Case DLL_.BPPreloader
            Get_DLL = bDLL_BPPreLoader
        Case DLL_.BPSubMain
            Get_DLL = bDLL_BPSubMain
    End Select

End Function

'change StdCall flags
'parameters - eDLL - flag
'           - bNewValue - new flag

Public Sub Let_DLL(ByVal eDLL As DLL_, sNewValue As String)

    Select Case eDLL
        Case DLL_.BaseAddress
            lBaseAddress = Val("&H" & sNewValue)
        Case DLL_.ExportedSymbols
            sExportedSymbols = sNewValue
        Case DLL_.EntryPointName
            sEntryPointName = sNewValue
        
        Case DLL_.ExportSymbols
            bExportSymbols = CBool(sNewValue)
        Case DLL_.LinkAsDll
            bDLL_LinkAsDLL = CBool(sNewValue)
        
        Case DLL_.FullLoading
            bDLL_FullLoading = CBool(sNewValue)
        Case DLL_.UsePreLoader
            bDLL_UsePreLoader = CBool(sNewValue)
        Case DLL_.CallMyDllMain
            bDLL_CallMyDllMain = CBool(sNewValue)
            
        Case DLL_.BPCallThunRTMain
            bDLL_BPCallThunRTMain = CBool(sNewValue)
        Case DLL_.BPPreloader
            bDLL_BPPreLoader = CBool(sNewValue)
        Case DLL_.BPSubMain
            bDLL_BPSubMain = CBool(sNewValue)
    End Select

End Sub

Public Function SaveSettingsToVariables(Optional bLoadForm As Boolean = False)
    
    If bLoadForm = True Then
        Load frmIn
        LoadSettings GLOBAL_, frmIn.pctSettings
        LoadSettings LOCAL_, frmIn.pctSettings
    End If
    
    With frmIn
      
        Let_DLL BaseAddress, .set_txtBaseAddress.Text
        Let_DLL EntryPointName, .set_txtEntryPoint.Text
        Let_DLL ExportedSymbols, .set_ctlExport.SelectedExports
  
        Let_DLL ExportSymbols, .set_chbExportSymbols.Value
        Let_DLL LinkAsDll, .set_chbCompileDll.Value
          
        Let_DLL FullLoading, .set_chbFullLoading.Value
        Let_DLL UsePreLoader, .set_chbUsePreLoader.Value
        Let_DLL CallMyDllMain, .set_chbCallMyDllMain.Value
  
        Let_DLL BPCallThunRTMain, .set_chbBPCallThunRTMain.Value
        Let_DLL BPPreloader, .set_chbBPPreLoader.Value
        Let_DLL BPSubMain, .set_chbBPSubMain.Value
      
    End With
    
    If bLoadForm = True Then
        Unload frmIn
    End If
    
End Function

Public Function SaveOtherSettings(eScope As SET_SCOPE) As Boolean
   'other controls that must be saved
   'this function is called from SaveSettings (modSaveLoadSettings)
   
   'Modified by Steve Tesch on 23-Aug-2006, to save exported function names to
   'sequenced Keys rather than a single Key.
   'Also changed the format of how the exported functions are stored in the .vbp
   'file.  Instead of function*module, I switched it the the more conventional
   'format of module.function.
   'Property Bag values are limited to 1024 characters when written to the .vbp
   'file, with the Key name and equal sign making up the first 14 characters.
   'This caused the exported function list to be truncated to 1010 characters,
   'limiting the number of exported functions that could be stored.  For large
   'projects, this meant you would have to re-check the box for some number of
   'functions, every time the project was re-opened.

   Dim sout() As String
   Dim lSeq As Long
   Dim sTemp As String
   
   If eScope = GLOBAL_ Then

      '--- global ---

   Else
      
      '--- local ---
      
      'In case this project had been saved under a previous version of this routine,
      'remove the original Key value if it exists.
      SaveSettingProject PLUGIN_NAME, frmIn.set_ctlExport.Name, vbNullString
      
      'Get the list of functions to eport.
      sTemp = frmIn.set_ctlExport.SelectedExports
      
      'Are there any functions to export?
      If sTemp <> "" Then
         'Split the list of functions to export into an output array.
         sout = Split(sTemp, frmIn.set_ctlExport.FuncSeparator)
         
         'Save the functions to export.
         'Step by 2 because the first entry is the function name and the second
         'entry is that functions module name.
         For lSeq = 1 To UBound(sout) Step 2
            'To make the sequence value attached to the Key name sequential
            '(i.e. not incremented by 2), it has to be calculated using:
            '(lSeq + 1) \ 2
            'For example, when lSeq = 1, we get Key 1, and when lSeq = 3,
            'we get Key 2, etc.
            'Save the value in module.function format.
            SaveSettingProject PLUGIN_NAME, _
                               frmIn.set_ctlExport.Name & CStr((lSeq + 1) \ 2), _
                               sout(lSeq) & "." & sout(lSeq - 1)
         Next lSeq
      End If
      
      'In case the user removed or unchecked some functions, we need to remove
      'any additional sequenced entries that may still exist in the .vbp file.
      'Initialize the sequence to the number of functions that were exported.
      If sTemp = "" Then
         lSeq = 0
      Else
         lSeq = (UBound(sout) + 1) \ 2
      End If
      
      'Loop through the potential sequenced Keys.
      Do
         'Increment the sequence.
         lSeq = lSeq + 1
         
         'See if this Key sequence exists.
         sTemp = GetSettingProject(PLUGIN_NAME, frmIn.set_ctlExport.Name & CStr(lSeq), vbNullString)
      
         'If it exists, delete it.
         If sTemp <> vbNullString Then _
            SaveSettingProject PLUGIN_NAME, frmIn.set_ctlExport.Name & CStr(lSeq), vbNullString
      Loop While sTemp <> vbNullString
   End If
   
   SaveOtherSettings = True
   
End Function
Public Function LoadOtherSettings(eScope As SET_SCOPE) As Boolean
   'other controls that must be inited
   'this function is called from LoadSettings (modSaveLoadSettings)

   'Modified by Steve Tesch on 23-Aug-2006, to load exported function names from
   'sequenced Keys rather than a single Key.
   'Property Bag values are limited to 1024 characters when written to the .vbp
   'file, with the Key name and equal sign making up the first 14 characters.
   'This caused the exported function list to be truncated to 1010 characters,
   'limiting the number of exported functions that could be stored.  For large
   'projects, this meant you would have to re-check the box for some number of
   'functions, every time the project was re-opened.

   Dim sIn As String
   Dim iPos As Integer
   Dim lSeq As Long
   Dim sTemp As String

   If eScope = GLOBAL_ Then
      
      '--- global ---
      
   Else
      
      '--- local ---
      
      With frmIn
         'Check the original Key first, in case this project had been saved under
         'a previous version of this routine.
         sIn = GetSettingProject(PLUGIN_NAME, .set_ctlExport.Name, vbNullString)
         
         If sIn = "" Then
            'Retrieve the sequenced Keys for exported functions.
            
            'Initialize the sequence.
            lSeq = 0
            
            'Loop until we get a blank one (i.e. there are no more).
            Do
               'Increment the sequence.
               lSeq = lSeq + 1
               
               'Read the Key.
               sTemp = GetSettingProject(PLUGIN_NAME, .set_ctlExport.Name & CStr(lSeq), vbNullString)
               
               If sTemp <> vbNullString Then
                  'Add the Key and a separator to the input string.
                  'Reverse the function name and module, with a separator between them,
                  'to match the format expected by the ExportList control.
                  'I specifically did not modify the ExportList control to use the
                  'module.function format, since I don't know if that control is used
                  'for other purposes.
                  iPos = InStr(sTemp, ".")
                  sIn = sIn & Right(sTemp, Len(sTemp) - iPos) & .set_ctlExport.FuncSeparator _
                            & Left(sTemp, iPos - 1) & .set_ctlExport.FuncSeparator
               End If
            Loop While sTemp <> vbNullString
            
            'Drop the last separator.
            sIn = Left(sIn, Len(sIn) - 1)
         End If
         
         'Pass the exported functions.
         .set_ctlExport.SelectedExports = sIn
         
         
         'some text must be in textboxes
         If Len(.set_txtBaseAddress.Text) = 0 Then modSettingsControls.DefaultTextBox_ComboBox GetScope(.set_txtBaseAddress), .set_txtBaseAddress
         If Len(.set_txtEntryPoint.Text) = 0 Then modSettingsControls.DefaultTextBox_ComboBox GetScope(.set_txtEntryPoint), .set_txtEntryPoint
      End With
   End If
       
   LoadOtherSettings = True
   
End Function

Public Sub SetOtherDefaultSettings(eScope As SET_SCOPE)
    'other controls that must be set to default
    'this function is called from SetDefaultSetings (modSaveLoadSettings)

    If eScope = GLOBAL_ Then
        
        '--- global ---
        
    Else
        
        '--- local ---
        
        frmIn.set_ctlExport.SelectedExports = ""
        
    End If
       
End Sub
